/**
 * 
 */
package edu.uoc.framework.generator.dataconfig;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.ManyToMany;
import javax.persistence.OneToMany;
import javax.persistence.OneToOne;
import javax.persistence.Transient;

import org.springframework.stereotype.Service;

import edu.uoc.framework.DTO.DataConfig;
import edu.uoc.framework.DTO.FieldConfig;
import edu.uoc.framework.common.ExceptionMessage;
import edu.uoc.framework.common.Level;
import edu.uoc.framework.exception.DataEmptyException;
import edu.uoc.framework.exception.DataFormatException;

/**
 * @author JBuch
 * 
 */
@Service
public class DataConfigGenerator {

	private List<DataConfig> listDataconfig;

	public DataConfigGenerator() {
		this.listDataconfig = new ArrayList<>();
	}

	public List<DataConfig> getAllDataConfig(List<Class<?>> listClass)
			throws DataEmptyException {

		DataConfig dco;

		if (listClass == null || listClass.size() < 1)
			throw new DataEmptyException(
					ExceptionMessage.EXC_DE_04_NOTCLASSLIST);

		for (Class<?> cls : listClass) {
			try {
				dco = getDataconfig(cls);
				listDataconfig.add(dco);
			} catch (NoSuchMethodException | SecurityException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (DataFormatException e) {
				e.printStackTrace();
			}
		}
		return listDataconfig;
	}

	public DataConfig getDataconfig(Class<?> cls) throws DataEmptyException,
			DataFormatException, NoSuchMethodException, SecurityException {

		DataConfig dataConfigObject = new DataConfig();
		List<FieldConfig> fieldTypeList = new ArrayList<>();
		Constructor<?> constructor;

		if (cls == null)
			throw new DataEmptyException(ExceptionMessage.EXC_DE_03_NOTCLASS);

		isEntityClass(cls);
		fieldTypeList = getListFieldConfig(cls);
		constructor = getConstructor(cls);

		dataConfigObject.setClassName(cls.getName());
		dataConfigObject.setCls(cls);
		dataConfigObject.setLevel(Level.LEVEL_01);
		dataConfigObject.setFieldList(fieldTypeList);
		dataConfigObject.setConstructor(constructor);

		return dataConfigObject;
	}

	private Constructor<?> getConstructor(Class<?> cls) {

		Constructor<?> constructorEntity = null;
		Constructor<?>[] constructorList = cls.getConstructors();

		for (Constructor<?> constructor : constructorList) {
			if (constructor.getParameterCount() == 0) {
				constructorEntity = constructor;
				break;
			}
		}

		return constructorEntity;
	}

	private List<FieldConfig> getListFieldConfig(Class<?> cls)
			throws NoSuchMethodException, SecurityException {

		List<FieldConfig> list = new ArrayList<>();
		FieldConfig fieldConfig;

		for (Field field : cls.getDeclaredFields()) {
			if (isCorrectField(field)) {
				fieldConfig = getFieldConfig(field, cls);
				list.add(fieldConfig);
			}
		}
		return list;
	}

	private FieldConfig getFieldConfig(Field field, Class<?> cls)
			throws NoSuchMethodException, SecurityException {

		FieldConfig fieldConfig = new FieldConfig();
		Column[] annotationColumn;
		Column column;
		Method setter;
		String name = field.getName();

		fieldConfig.setName(name);

		fieldConfig.setField(field);

		annotationColumn = field.getAnnotationsByType(Column.class);
		if (annotationColumn.length > 0) {

			column = annotationColumn[0];

			fieldConfig.setNullable(column.nullable());
			fieldConfig.setLength(column.length());
			fieldConfig.setPrecision(column.precision());
			fieldConfig.setScale(column.scale());

		}

		annotationColumn = field.getAnnotationsByType(Column.class);
		if (annotationColumn.length > 0) {
			
		}
		
		setter = cls.getMethod("set" + Character.toUpperCase(name.charAt(0))
				+ name.substring(1), field.getType());
		fieldConfig.setSetter(setter);

		return fieldConfig;
	}

	private boolean isCorrectField(Field field) {
		
		boolean correctField = true;
		
		if(field.getModifiers() == Modifier.TRANSIENT ||
				field.getAnnotationsByType(Id.class).length > 0 ||
				field.getAnnotationsByType(Transient.class).length > 0 ||
				isMappedBy(field))
			correctField = false;
		
		return correctField;	
	}
	
	/*Si el field es mapped by vol dir que 
	 * es una relacio bidfireccional i no hem de crear el camp.
	 */
	private boolean isMappedBy(Field field) {
		
		boolean mappedbyField = false;
		OneToMany oneToMany;
		OneToOne oneToOne;
		ManyToMany manyToMany;
		
		OneToMany[] oneToManyArray;
		OneToOne[] oneToOneArray;
		ManyToMany[] manyToManyArray;
		
		oneToManyArray = field.getAnnotationsByType(OneToMany.class);
		oneToOneArray = field.getAnnotationsByType(OneToOne.class);
		manyToManyArray = field.getAnnotationsByType(ManyToMany.class);
		
		if (oneToManyArray.length > 0) {
			oneToMany = oneToManyArray[0];
			mappedbyField = oneToMany.mappedBy() != null && oneToMany.mappedBy().length()>0;	
		}else if (oneToOneArray.length > 0) {
			oneToOne = oneToOneArray[0];
			mappedbyField = oneToOne.mappedBy() != null && oneToOne.mappedBy().length()>0;
		}else if (manyToManyArray.length > 0) {
			manyToMany = manyToManyArray[0];
			mappedbyField = manyToMany.mappedBy() != null && manyToMany.mappedBy().length()>0;
		}

		return mappedbyField;
	}

	private void isEntityClass(Class<?> cls) throws DataFormatException{		
		Entity[] annotationEntity = cls.getAnnotationsByType(Entity.class);
		if(annotationEntity == null || annotationEntity.length < 1) 
			throw new DataFormatException(ExceptionMessage.EXC_DF_01_NOTENTITY);
	}
	
	public List<DataConfig> getListDataconfig() {
		return listDataconfig;
	}

	public void setListDataconfig(List<DataConfig> listDataconfig) {
		this.listDataconfig = listDataconfig;
	}
}
