/**
 * 
 */
package edu.uoc.framework.generator.entity;

import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import edu.uoc.framework.DTO.DataConfig;
import edu.uoc.framework.DTO.FieldConfig;
import edu.uoc.framework.common.ExceptionMessage;
import edu.uoc.framework.common.Level;
import edu.uoc.framework.exception.DataEmptyException;
import edu.uoc.framework.exception.DataFormatException;
import edu.uoc.framework.exception.TypeAcceptException;

/**
 * @author JBuch
 * 
 */
@Service
public class DataEntityGenerator {

	private List<Object> listDataEntity;
	private final static int MAX_ENTITYS = 20;
	private final static int MAX_SUB_ENTITYS = 5;

	private List<DataConfig> listDataConfig;

	@Autowired
	DataFieldGenerator dataFieldGenerator;

	public DataEntityGenerator() {
		this.listDataEntity = new ArrayList<>();
	}

	/*
	 * Generem per cada DataConfig 10 entitys i les afegim a la llista
	 */
	public List<Object> getEntityList(List<DataConfig> listDataConfig)
			throws DataEmptyException {

		Collection<Object> temporalEntityList;

		if (listDataConfig == null || listDataConfig.size() < 1)
			throw new DataEmptyException(ExceptionMessage.EXC_DE_05_DATACONFIG);

		this.listDataConfig = listDataConfig;

		for (DataConfig dataConfig : listDataConfig) {
			temporalEntityList = getEntitysByDataConfig(dataConfig, MAX_ENTITYS);
			listDataEntity.addAll(temporalEntityList);
		}

		return listDataEntity;
	}

	private Collection<Object> getEntitysByDataConfig(DataConfig dataConfig,
			int maxEntitys) {

		Collection<Object> temporalEntityList = new ArrayList<>();
		Object entity;

		for (int i = 0; i < maxEntitys; i++) {
			try {
				entity = getEntityByDataConfig(dataConfig);
				if (entity != null)
					temporalEntityList.add(entity);
			} catch (DataFormatException e) {
				System.err.println(e.getMessage());
			} catch (Exception e) {
				System.err.println(e.getMessage());
			}
		}
		return temporalEntityList;
	}

	private Object getEntityByDataConfig(DataConfig dataConfig)
			throws DataFormatException, InstantiationException,
			IllegalAccessException, IllegalArgumentException,
			InvocationTargetException, NoSuchMethodException, SecurityException {

		Object entity = null;
		Constructor<?> constructorEntity = null;

		if (dataConfig == null || dataConfig.getCls() == null
				|| dataConfig.getFieldList().size() < 1
				|| dataConfig.getConstructor() == null)
			throw new DataFormatException(
					ExceptionMessage.EXC_DF_03_NOTCORRECTDATACONFIG);

		constructorEntity = dataConfig.getConstructor();

		// En primer lloc instanciem el objecte de la clase
		if (constructorEntity != null)
			entity = constructorEntity.newInstance();
		// Creem les dades del objecte
		if (entity != null) {
			loadEntityField(entity, dataConfig.getFieldList(),
					dataConfig.getLevel());
		}
		return entity;
	}

	private void loadEntityField(Object entity, List<FieldConfig> fieldList,
			int level) throws IllegalArgumentException, IllegalAccessException {

		Method setter;
		Object value;

		for (FieldConfig field : fieldList) {

			setter = field.getSetter();
			try {
				value = dataFieldGenerator.getValue(field.getField().getType(),
						field.isNullable(), field.getLength(),
						field.getPrecision(), field.getScale());
			} catch (TypeAcceptException e) {
				value = null;
				System.err.println(e.getMessage());
			}

			// Tenim que el value potser un objecte Type aixo vol dir que
			// podriem tenir una relacio entre Entitys.
			if (value != null && value instanceof Type
					&& level == Level.LEVEL_01) {
				try {
					value = getSubEntity(field.getField());
				} catch (TypeAcceptException e) {
					value = null;
					System.err.println(e.getMessage());
				}
			}

			try {
				if (setter != null && !(value instanceof Type)) {
					setter.invoke(entity, value);
				}
			} catch (InvocationTargetException | IllegalArgumentException e) {
				System.err.println(e.getMessage());
			}
		}
	}

	/*
	 * Comprovarem que el value(field.getField().getType()) sigui una colleccio
	 * o no: Si es un Collection nececitem el tipus generic de Collection. Si no
	 * es un Collection llavors ja tenim el tipus de Entity. Amb els dos casos
	 * obtindrem un tipus de Objecte i comprovarem si aquest esta dins dels
	 * DataConfig: Si esta dins procedirem a cear les entitys de la relacio. Si
	 * no esta dins el type del objecte no es acceptat pel generador.
	 */
	private Object getSubEntity(Field field) throws TypeAcceptException {

		DataConfig subEntityDataConfig;
		Type type;
		Object value = null;
		boolean isCollection = Collection.class.isAssignableFrom(field
				.getType());

		type = isCollection ? ((ParameterizedType) field.getGenericType())
				.getActualTypeArguments()[0] : field.getType();

		subEntityDataConfig = getDataconfig(type);

		if (subEntityDataConfig == null) {
			throw new TypeAcceptException(
					ExceptionMessage.EXC_TY_01_NOTCORRECTDATACONFIG);
		} else if (isCollection) {
			value = getEntitysByDataConfig(subEntityDataConfig, MAX_SUB_ENTITYS);
		} else {
			try {
				value = getEntityByDataConfig(subEntityDataConfig);
			} catch (Exception e) {
				System.err.println(e.getMessage());
			}
		}
		return value;
	}

	private DataConfig getDataconfig(Type type) {

		DataConfig subDataConfig = null;

		for (DataConfig dataConfig : listDataConfig) {
			if (dataConfig.getCls().equals(type)) {
				subDataConfig = dataConfig.clone();
				subDataConfig.setLevel(Level.LEVEL_02);
				break;
			}
		}

		return subDataConfig;
	}

	public List<Object> getListDataEntity() {
		return listDataEntity;
	}

	public void setListDataEntity(List<Object> listDataEntity) {
		this.listDataEntity = listDataEntity;
	}

}
