/**
 * 
 */
package edu.uoc.framework.generator.entity.RandomGenerator;

import java.sql.Date;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.Random;

import org.springframework.stereotype.Component;

import edu.uoc.framework.common.ConstantsType;
import edu.uoc.framework.common.ExceptionMessage;
import edu.uoc.framework.exception.TypeAcceptException;

/**
 * @author JBuch
 * 
 */
@Component
public class RandomGenerator {

	private static Random random = new Random();
	private final static String LIST_CHAR = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";

	public Object getRandomValue(int type, boolean nullable, int length,
			int precision, int scale) throws TypeAcceptException {

		Object value = null;
		// Si l'objecte es nullable generem un boolean que indicara si sera null
		// o no.
		// Si generem un false el objecte sera null
		if (!nullable || random.nextBoolean()) {
			switch (type) {
			case ConstantsType.STRING_TYPE:
				value = getValueRandomString(length);
				break;
			case ConstantsType.BOOLEAN_PR_TYPE:
				value = random.nextBoolean();
				break;
			case ConstantsType.BYTE_PR_TYPE:
				value = getByte();
				break;
			case ConstantsType.SHORT_PR_TYPE:
				value = (short) random.nextInt(1 << 16);
				break;
			case ConstantsType.CHAR_PR_TYPE:
				value = LIST_CHAR.charAt(random.nextInt(LIST_CHAR.length()));
				break;
			case ConstantsType.INT_PR_TYPE:
				value =  random.nextInt();
				break;
			case ConstantsType.LONG_PR_TYPE:
				value = random.nextLong();
				break;
			case ConstantsType.FLOAT_PR_TYPE:
				value = random.nextFloat()+random.nextInt();
				break;
			case ConstantsType.DOUBLE_PR_TYPE:
				value = random.nextDouble()+random.nextInt();
				break;
			// Objectes
			case ConstantsType.BOOLEAN_TYPE:
				value = new Boolean(random.nextBoolean());
				break;
			case ConstantsType.BYTE_TYPE:
				value = new Byte(getByte());
				break;
			case ConstantsType.SHORT_TYPE:
				value = new Short((short)random.nextInt(1 << 16));
				break;
			case ConstantsType.CHARACTER_TYPE:
				value = new Character(LIST_CHAR.charAt(random.nextInt(LIST_CHAR.length())));
				break;
			case ConstantsType.INTEGER_TYPE:
				value = new Integer(random.nextInt());
				break;
			case ConstantsType.LONG_TYPE:
				value = new Long(random.nextLong());
				break;
			case ConstantsType.FLOAT_TYPE:
				value = new Float(random.nextFloat()+random.nextInt());
				break;
			case ConstantsType.DOUBLE_TYPE:
				value = new Double(random.nextDouble()+random.nextInt());
				break;
			// Dates java.sql.Date
			case ConstantsType.DATE_SQL_TYPE:
				value = getDate();
				break;
			case ConstantsType.TIME_SQL_TYPE:
				value = getTime();
				break;
			case ConstantsType.TIMESTAMP_SQL_TYPE:
				value = getTimestamp();
				break;
			default:
				throw new TypeAcceptException(
						ExceptionMessage.EXC_TY_01_NOTCORRECTDATACONFIG);
			}
		}
		return value;
	}

	private Timestamp getTimestamp() {
		Long ms = -946771200000L + (Math.abs(random.nextLong()) % (70L * 365 * 24 * 60 * 60 * 1000));
		Timestamp time = new Timestamp(ms);
		time.setNanos(0);
		return time;
	}

	private Time getTime() {
		Long ms = -946771200000L + (Math.abs(random.nextLong()) % (70L * 365 * 24 * 60 * 60 * 1000));
		return new Time(ms);
	}

	private Date getDate() {
		Long ms = -946771200000L + (Math.abs(random.nextLong()) % (70L * 365 * 24 * 60 * 60 * 1000));
		return new Date(ms);
	}

	private byte getByte() {
		
		byte[] byteArray = new byte[1];
		random.nextBytes(byteArray);
		
		return byteArray[0];
	}

	//El tamany i el contingut del String tenen que ser aleatoris
	private String getValueRandomString(int length) {
		int sizeString = random.nextInt(length);
		StringBuilder sb = new StringBuilder(length);
		for (int i = 0; i < sizeString; i++)
			sb.append(LIST_CHAR.charAt(random.nextInt(LIST_CHAR.length())));
		return sb.toString();
	}
}
