/**
 * 
 */
package edu.uoc.framework.read;

import java.io.File;
import java.net.URISyntaxException;
import java.net.URL;
import java.util.ArrayList;

import org.springframework.stereotype.Component;

import edu.uoc.framework.common.ExceptionMessage;
import edu.uoc.framework.exception.DataEmptyException;

/**
 * @author Dave Dopson
 * @version 1.2
 * @Modified Joan Buch, metode extret d'internet modificat per a que s'adapti a les necessitats del framework
 * Classe que lleigeix les 'Class' d'un 'package'
 */
@Component
public final class ReadPackageClass {
	
	public ArrayList<Class<?>> getClassesForPackage(Package pkg) throws DataEmptyException {
		
		if(pkg == null) throw new DataEmptyException(ExceptionMessage.EXC_DE_02_NOTPACKAGE);
	    String pkgname = pkg.getName();
	    ArrayList<Class<?>> classes = new ArrayList<Class<?>>();
	    // Get a File object for the package
	    File directory = null;
	    String relPath = pkgname.replace('.', '/');
	    URL resource = ClassLoader.getSystemClassLoader().getResource(relPath);
	    if (resource == null) {
	        throw new RuntimeException("No resource for " + relPath);
	    }

	    try {
	        directory = new File(resource.toURI());
	    } catch (URISyntaxException e) {
	        throw new RuntimeException(pkgname + " (" + resource + ") does not appear to be a valid URL / URI.  Strange, since we got it from the system...", e);
	    } catch (IllegalArgumentException e) {
	        directory = null;
	    }
	    if (directory != null && directory.exists()) {
	        // Get the list of the files contained in the package
	        String[] files = directory.list();
	        for (int i = 0; i < files.length; i++) {
	            // we are only interested in .class files
	            if (files[i].endsWith(".class")) {
	                // removes the .class extension
	                String className = pkgname + '.' + files[i].substring(0, files[i].length() - 6);
	                try {
	                    classes.add(Class.forName(className));
	                } 
	                catch (ClassNotFoundException e) {
	                    throw new RuntimeException("ClassNotFoundException loading " + className);
	                }
	            }
	        }
	    }
	    return classes;
	}
}
